package pcc.chemicraft;

import java.util.ArrayList;
import java.util.HashMap;

import net.minecraft.item.ItemStack;
import pcc.chemicraft.system.ChemiCraftCraftingManager;
import pcc.chemicraft.tileentity.TileEntityPyrolysisTable;
import pcc.chemicraft.util.ChemicalNBTRecipe;
import pcc.chemicraft.util.ICompoundHandler;
import pcc.chemicraft.util.MaterialRecipe;


/**
 * ChemiCraftのAPI
 * 基本的にAddonはこのクラスのインスタンスを使う
 * @author mozipi
 *
 */
public class ChemiCraftAPI {

	/**
	 * Instance of the ChemiCraftAPI.
	 */
	private static ChemiCraftAPI instance = new ChemiCraftAPI();


	/**
	 * List of compounds names.
	 */
	private ArrayList<String> compoundsNameList = new ArrayList<String>();


	/**
	 * List of compounds names(Some kind of language).
	 */
	private ArrayList<String> compoundsLangNameList = new ArrayList<String>();


	/**
	 * List of compounds the language names.
	 */
	private ArrayList<String> compoundsLangList = new ArrayList<String>();


	/**
	 * List of compounds handlers.
	 */
	private ArrayList<ICompoundHandler> compoundHandlers = new ArrayList<ICompoundHandler>();


	/**
	 * 化合物の文字列をダメージ値に変換します。
	 */
	private HashMap<String, Integer> compoundHash = new HashMap<String, Integer>();


	/**
	 * List of item name of handler to compounds.
	 */
	private ArrayList<String> compoundHandlerItemNames = new ArrayList<String>();


	/**
	 * 化合台の原子の種類のリスト
	 */
	private ArrayList<String[]> chemicalCombinationAtoms = new ArrayList<String[]>();


	/**
	 * 化合台の原子の数のリスト
	 */
	private ArrayList<Integer[]> chemicalCombinationAmounts = new ArrayList<Integer[]>();


	/**
	 * 化合台の結果のリスト
	 */
	private ArrayList<ItemStack> chemicalCombinationResult = new ArrayList<ItemStack>();


	/**
	 * 素材製作台のレシピクラス
	 */
	private ArrayList<MaterialRecipe> materialRecipe = new ArrayList<MaterialRecipe>();


	/**
	 * ChemiCraftの化学作業台類のレシピのマネージャー
	 */
	private ChemiCraftCraftingManager chemiCraftCraftingManager = new ChemiCraftCraftingManager();



	/**
	 * 鉱石名リスト
	 */
	private ArrayList<String> atomsOreNameList = new ArrayList<String>();


	/**
	 * 鉱石IDリスト
	 */
	private ArrayList<Integer> atomsOreIDList = new ArrayList<Integer>();


	/**
	 * 鉱石メタデータリスト
	 */
	private ArrayList<Integer> atomsOreMetaList = new ArrayList<Integer>();


	/**
	 * 鉱石量リスト
	 */
	private ArrayList<Integer> atomsOreSizeList = new ArrayList<Integer>();


	/**
	 * 鉱石頻度リスト
	 */
	private ArrayList<Integer> atomsOreFrequencyList = new ArrayList<Integer>();


	/**
	 * 鉱石Y座標リスト
	 */
	private ArrayList<Integer> atomsOrePosYList = new ArrayList<Integer>();



	/**
	 * ツール&武器作成台の素材一覧のリスト
	 */
	private ArrayList<ItemStack[]> toolAndWeaponMaterials = new ArrayList<ItemStack[]>();


	/**
	 * ツール&武器作成台の結果のリスト
	 */
	private ArrayList<ItemStack> toolAndWeaponResult = new ArrayList<ItemStack>();


	/**
	 * ツール&武器作成台の不定形であるか
	 */
	private ArrayList<Boolean> toolAndWeaponSharpless = new ArrayList<Boolean>();



	/**
	 * add compound.
	 * @param name compound name.
	 */
	public void addCompound(String name){
		addLangCompound("", name, "");
	}



	/**
	 * add compound corresponding to the language.
	 * @param lang Language to the corresponding
	 * @param englishName compound name
	 * @param langName compound name(specified language)
	 */
	public void addLangCompound(String lang, String englishName, String langName){
		compoundsNameList.add(englishName);
		compoundsLangNameList.add(langName);
		compoundsLangList.add(lang);
		compoundHash.put(englishName, compoundHash.size());
	}



	public int getCompound(String key){
		if(compoundHash.get(key) != null){
			return compoundHash.get(key);
		} else {
			return -1;
		}
	}



	/**
	 * setting compound handler.
	 * @param handlerItemName
	 * @param compoundHandler
	 */
	public void settingCompoundHandler(String handlerItemName, ICompoundHandler compoundHandler){
		compoundHandlers.add(compoundHandler);
		compoundHandlerItemNames.add(handlerItemName);
	}



	/**
	 * 化合レシピを追加します。materialの要素数は0<= n <= 16にしてください。
	 * @param material 素材
	 * @param result 結果
	 */
	public void addChemicalCombinationRecipe(String[] atoms, Integer[] amounts, ItemStack result){
		chemicalCombinationAtoms.add(atoms);
		chemicalCombinationAmounts.add(amounts);
		chemicalCombinationResult.add(result);
	}



	/**
	 * 分解台のレシピを追加します
	 * @param material 素材
	 * @param results 原子の元素番号の配列
	 * @param stacks 原子のできる数の配列
	 */
	public void addDecompositionRecipe(ItemStack material, int[] results, int[] stacks) {
		ItemStack[] itemstacks = new ItemStack[results.length];
		for (int i = 0; i < itemstacks.length; i++) {
			itemstacks[i] = new ItemStack(ChemiCraft.instance.itemAtoms,stacks[i], results[i]);
		}
		TileEntityPyrolysisTable.addRecipe(material, itemstacks);
	}



	/**
	 * 分解台の燃料を追加します
	 * @param itemstack 燃料のItemStack
	 * @param burnTime 燃焼時間(tick * rate)
	 */
	public void addDecompositionFuel(ItemStack itemstack, int burnTime) {
		TileEntityPyrolysisTable.addFuel(itemstack, burnTime);
	}



	/**
	 * ツール&武器作成台のレシピを追加します
	 * @param materials 素材
	 * @param result 結果
	 */
	public void addToolAndWeaponRecipe(ItemStack[] materials, ItemStack result) {
		toolAndWeaponMaterials.add(materials);
		toolAndWeaponResult.add(result);
		toolAndWeaponSharpless.add(false);
	}


	/**
	 * ツール&武器作成台の不定形レシピを追加します
	 * @param materials 素材
	 * @param result 結果
	 */
	public void addSharplessToolAndWeaponRecipe(ItemStack[] materials, ItemStack result) {
		toolAndWeaponMaterials.add(materials);
		toolAndWeaponResult.add(result);
		toolAndWeaponSharpless.add(true);
	}



	/**
	 * 素材作成台の不定形レシピを追加します
	 * @param materials 素材
	 * @param result 結果
	 * @param nbtRecipe NBT(Nullの場合はなし)
	 */
	public void addSharplessMaterialRecipe(ItemStack[] materials, ItemStack result, ChemicalNBTRecipe nbtRecipe){
		materialRecipe.add(new MaterialRecipe(result, materials, nbtRecipe, true));
	}



	/**
	 * 素材作成台のレシピを追加します
	 * @param materials 素材
	 * @param result 結果
	 * @param nbtRecipe NBT(Nullの場合はなし)
	 */
	public void addMaterialRecipe(ItemStack[] materials, ItemStack result, ChemicalNBTRecipe nbtRecipe){
		materialRecipe.add(new MaterialRecipe(result, materials, nbtRecipe, false));
	}



	/**
	 * 鉱石を追加します
	 * @param name 鉱石名
	 * @param id 鉱石ID
	 * @param meta 鉱石メタデータ
	 * @param size 一度に生成される量
	 * @param frequency 生成頻度
	 * @param posY Y座標
	 */
	public void addAtomsOre(String name, int id, int size, int frequency, int posY){
		addAtomsOre(name, id, 0, size, frequency, posY);
	}


	public void addAtomsOre(String name, int id, int meta, int size, int frequency, int posY){
		atomsOreNameList.add(name);
		atomsOreIDList.add(id);
		atomsOreMetaList.add(meta);
		atomsOreSizeList.add(size);
		atomsOreFrequencyList.add(frequency);
		atomsOrePosYList.add(posY);
	}


	
	//以下システム関連//////////////////////////////////////////////////////

	public ArrayList<ICompoundHandler> getCompoundHandler(){
		compoundHandlers.trimToSize();
		return compoundHandlers;

	}



	public ArrayList<String> getCompoundHandlerItemName(){
		compoundHandlerItemNames.trimToSize();
		return compoundHandlerItemNames;
	}



	public ArrayList<String> getCompoundsName(){
		compoundsNameList.trimToSize();
		return compoundsNameList;
	}



	public ArrayList<String> getCompoundsLangName(){
		compoundsLangNameList.trimToSize();
		return compoundsLangNameList;
	}



	public ArrayList<String> getCompoundsLang(){
		compoundsLangList.trimToSize();
		return compoundsLangList;
	}



	public ArrayList<String[]> getChemicalCombinationAtoms(){
		return chemicalCombinationAtoms;
	}



	public ArrayList<Integer[]> getChemicalCombinationAmounts(){
		return chemicalCombinationAmounts;
	}



	public ArrayList<ItemStack> getChemicalCombinationResult(){
		return chemicalCombinationResult;
	}



	public ArrayList<MaterialRecipe> getMaterialRecipe(){
		return materialRecipe;
	}



	public ArrayList<ItemStack[]> getToolAndWeaponMaterials() {
		return toolAndWeaponMaterials;
	}



	public ArrayList<ItemStack> getToolAndWeaponResult() {
		return toolAndWeaponResult;
	}



	public ArrayList<Boolean> getToolAndWeaponSharpless() {
		return toolAndWeaponSharpless;
	}



	public ChemiCraftCraftingManager getCraftingManager(){
		return chemiCraftCraftingManager;
	}



	public ArrayList<String> getAtomsOreNameList(){
		return atomsOreNameList;
	}



	public ArrayList<Integer> getAtomsOreIDList(){
		return atomsOreIDList;
	}



	public ArrayList<Integer> getAtomsOreMetaList(){
		return atomsOreMetaList;
	}



	public ArrayList<Integer> getAtomsOreSizeList(){
		return atomsOreSizeList;
	}



	public ArrayList<Integer> getAtomsOreFrequencyList(){
		return atomsOreFrequencyList;
	}



	public ArrayList<Integer> getAtomsOrePosYList(){
		return atomsOrePosYList;
	}



	public static ChemiCraftAPI instance(){
		return instance;
	}

}
